@@ -2,7 +2,7 @@
 # Maintainer     : Pierre Roudier (pierre.roudier@landcare.nz);
 # Contributions  : Tomislav Hengl (tom.hengl@wur.nl); Dylan Beaudette (debeaudette@ucdavis.edu); 
 # Status         : tested
-# Note           : in the case of gridded data, bounding box and cell size are estimated by the program (raster / FWTools);
+# Note           : in the case of gridded data, bounding box and cell size are estimated by the program (raster / GDAL);
 
 
 reproject.SpatialPoints <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts), ...) {
@@ -26,10 +26,10 @@ reproject.RasterLayer <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
     names(res) <- names(obj)
   } else {
   
-  if(program=="FWTools"){
+  if(program=="GDAL"){
     gdalwarp <- get("gdalwarp", envir = plotKML.opts)
   
-    # look for FWTools path:  
+    # look for GDAL path:  
     if(nchar(gdalwarp)==0){
       plotKML.env(silent = FALSE, show.env = FALSE)
       gdalwarp <- get("gdalwarp", envir = plotKML.opts)
@@ -45,12 +45,13 @@ reproject.RasterLayer <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
       if(method == "ngb") { method <- "near" }
         writeRaster(obj, paste(tf, ".tif", sep=""), overwrite=TRUE, NAflag=NAflag)
         # resample to WGS84 system:
+        message(paste("Using gdalwarp function:", gdalwarp))
         message(paste("Reprojecting to", CRS, "..."))
         system(paste(gdalwarp, " ", tf, ".tif", " -t_srs \"", CRS, "\" ", tf, "_ll.tif -dstnodata \"", NAflag, "\" ", " -r ", method, sep=""), show.output.on.console = show.output.on.console)
         res <- raster(paste(tf, "_ll.tif", sep=""), silent = TRUE)
         names(res) <- names(obj)
       } else {
-        stop("Could not locate FWTools. See 'plotKML.env()' for more info.") }
+        stop("Could not locate GDAL. See 'plotKML.env()' for more info.") }
   }
   }
   
@@ -60,6 +61,11 @@ reproject.RasterLayer <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
 
 reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts), tmp.file = TRUE, program = "raster", NAflag = get("NAflag", envir = plotKML.opts), show.output.on.console = FALSE, ...) {
 
+  ## convert all character vectors to factors:
+  for(j in 1:ncol(obj)){ 
+    if(is.character(obj@data[,j])){ obj@data[,j] <- as.factor(obj@data[,j]) }
+  }
+
   if(program=="raster"){
 
     # if multiple layers:
@@ -82,7 +88,7 @@ reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
     else {
       r <- raster(obj)
       if(is.factor(obj@data[,1])){
-        r <- raster::as.factor(r)
+        if(!is.raster(r)){ r <- raster::as.factor(r) }
         message(paste("Reprojecting to", CRS, "..."))
         res <- as(raster::projectRaster(r, crs = CRS, method = "ngb"), "SpatialGridDataFrame")
       } else {
@@ -92,24 +98,24 @@ reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
       names(res) <- names(obj)
     }
     
-    # fix factor-type objects:
+    # try to fix factor-type objects:
     for(j in 1:ncol(obj)){
       if(is.factor(obj@data[,j])){
         # copy levels:
         res@data[,j] <- as.factor(res@data[,j])
-        levels(res@data[,j]) = levels(as.factor(paste(obj@data[,j])))
+        try( levels(res@data[,j]) <- levels(obj@data[,j]), silent = TRUE )
       }
     }
   }
   
-  if(program=="FWTools"){
+  if(program=="GDAL"){
   gdalwarp <- get("gdalwarp", envir = plotKML.opts)
-  
-  # look for FWTools path:  
+  # look for GDAL path if missing:  
   if(nchar(gdalwarp)==0){
     plotKML.env(silent = FALSE)
     gdalwarp <- get("gdalwarp", envir = plotKML.opts)
   }
+  message(paste("Using gdalwarp function:", gdalwarp))
   
   if(!nchar(gdalwarp)==0){
   
@@ -126,7 +132,11 @@ reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
         if(is.factor(obj@data[,j])){
           x <- obj[j]
           x@data[,1] <- as.integer(x@data[,1])
-          writeGDAL(x, paste(tf, ".tif", sep=""), "GTiff")
+          if(max(x@data[,1], na.rm=TRUE)<254){
+            writeGDAL(x, paste(tf, ".tif", sep=""), "GTiff", mvFlag = 255, type="Byte")
+          } else {
+            writeGDAL(x, paste(tf, ".tif", sep=""), "GTiff")
+          }
         }        
         else {
           writeGDAL(obj[j], paste(tf, ".tif", sep=""), "GTiff")
@@ -140,7 +150,7 @@ reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
         else {
           system(paste(gdalwarp, ' ', tf, '.tif', ' -t_srs \"', CRS, '\" ', tf, '_ll.tif -dstnodata \"', NAflag, '\" -r bilinear', sep=""), show.output.on.console = show.output.on.console)
         }
-        # read images back to R:
+        ## read images back to R:
         if(j==1){
           res <- readGDAL(paste(tf, "_ll.tif", sep=""), silent = TRUE)
           names(res) <- names(obj)[j]
@@ -149,10 +159,10 @@ reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
           res@data[names(obj)[j]] <- readGDAL(paste(tf, "_ll.tif", sep=""), silent = TRUE)$band1
         }
         
-        # reformat to the original factors:
+        ## reformat to the original factors:
           if(is.factor(obj@data[,j])){
             res@data[,j] <- as.factor(res@data[,j])
-            levels(res@data[,j]) = levels(as.factor(paste(obj@data[,j])))
+            try( levels(res@data[,j]) <- levels(obj@data[,j]) , silent = TRUE)
         }
         unlink(paste(tf, ".tif", sep=""))
         unlink(paste(tf, "_ll.tif", sep=""))        
@@ -160,7 +170,7 @@ reproject.SpatialGrid <- function(obj, CRS = get("ref_CRS", envir = plotKML.opts
   } 
   
   else {
-    stop("Could not locate FWTools. See 'plotKML.env()' for more info.") 
+    stop("Could not locate GDAL. See 'plotKML.env()' for more info.") 
   }
   
   } 
